"use strict";
/**
 * @license
 * Copyright (c) 2016 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var fs = require("mz/fs");
var path = require("path");
var url = require("url");
var args_1 = require("./args");
var compile_middleware_1 = require("./compile-middleware");
var start_server_1 = require("./start_server");
var commandLineArgs = require("command-line-args");
var commandLineUsage = require("command-line-usage");
function run() {
    return __awaiter(this, void 0, void 0, function () {
        var argsWithHelp, cliOptions, proxyArgs, options, serverInfos, mainlineServer, urls, urls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    argsWithHelp = args_1.args.concat({
                        name: 'help',
                        description: 'Shows this help message',
                        type: Boolean,
                    });
                    try {
                        cliOptions = commandLineArgs(argsWithHelp);
                    }
                    catch (e) {
                        printUsage(argsWithHelp);
                        return [2 /*return*/];
                    }
                    proxyArgs = {
                        path: cliOptions['proxy-path'],
                        target: cliOptions['proxy-target']
                    };
                    options = {
                        root: cliOptions.root,
                        compile: cliOptions.compile,
                        moduleResolution: cliOptions['module-resolution'],
                        port: cliOptions.port,
                        hostname: cliOptions.hostname,
                        open: cliOptions.open,
                        browser: cliOptions['browser'],
                        openPath: cliOptions['open-path'],
                        componentDir: cliOptions['component-dir'],
                        componentUrl: cliOptions['component-url'],
                        packageName: cliOptions['package-name'],
                        npm: cliOptions['npm'],
                        protocol: cliOptions['protocol'],
                        keyPath: cliOptions['key'],
                        certPath: cliOptions['cert'],
                        pushManifestPath: cliOptions['manifest'],
                        proxy: proxyArgs.path && proxyArgs.target && proxyArgs,
                        allowOrigin: cliOptions['allow-origin'],
                    };
                    if (cliOptions.help) {
                        printUsage(argsWithHelp);
                        return [2 /*return*/];
                    }
                    if (cliOptions.version) {
                        console.log(getVersion());
                        return [2 /*return*/];
                    }
                    if (typeof cliOptions['compile-cache'] === 'number') {
                        console.log("compile cache set to " + cliOptions['compile-cache']);
                        compile_middleware_1.babelCompileCache['max'] = cliOptions['compile-cache'];
                    }
                    return [4 /*yield*/, start_server_1.startServers(options)];
                case 1:
                    serverInfos = _a.sent();
                    if (serverInfos.kind === 'mainline') {
                        mainlineServer = serverInfos;
                        urls = start_server_1.getServerUrls(options, mainlineServer.server);
                        console.log("Files in this directory are available under the following URLs\n    applications: " + url.format(urls.serverUrl) + "\n    reusable components: " + url.format(urls.componentUrl) + "\n  ");
                    }
                    else {
                        urls = start_server_1.getServerUrls(options, serverInfos.control.server);
                        console.log("Started multiple servers with different variants:\n    More info here: " + url.format(urls.serverUrl));
                    }
                    return [2 /*return*/, serverInfos];
            }
        });
    });
}
exports.run = run;
function printUsage(options) {
    var usage = commandLineUsage([{
            header: 'A development server for Polymer projects',
            title: 'polyserve',
            optionList: options,
        }]);
    console.log(usage);
}
function getVersion() {
    var packageFilePath = path.resolve(__dirname, '../package.json');
    var packageFile = fs.readFileSync(packageFilePath).toString();
    var packageJson = JSON.parse(packageFile);
    var version = packageJson['version'];
    return version;
}
