"use strict";
/**
 * @license
 * Copyright (c) 2015 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var assert = require("assert");
var escapeHtml = require("escape-html");
var express = require("express");
var fs = require("mz/fs");
var path = require("path");
var path_transformers_1 = require("polymer-build/lib/path-transformers");
var send = require("send");
// TODO: Switch to node-http2 when compatible with express
// https://github.com/molnarg/node-http2/issues/100
var http = require("spdy");
var compile_middleware_1 = require("./compile-middleware");
var config_1 = require("./config");
var custom_elements_es5_adapter_middleware_1 = require("./custom-elements-es5-adapter-middleware");
var make_app_1 = require("./make_app");
var open_browser_1 = require("./util/open_browser");
var push_1 = require("./util/push");
var tls_1 = require("./util/tls");
var compression = require("compression");
var cors = require("cors");
var httpProxy = require('http-proxy-middleware');
function applyDefaultServerOptions(options) {
    var withDefaults = Object.assign({}, options);
    Object.assign(withDefaults, {
        port: options.port || 0,
        hostname: options.hostname || 'localhost',
        root: path.resolve(options.root || '.'),
        compile: options.compile || 'auto',
        certPath: options.certPath || 'cert.pem',
        keyPath: options.keyPath || 'key.pem',
        componentDir: config_1.getComponentDir(options),
        componentUrl: options.componentUrl || 'components'
    });
    withDefaults.packageName = config_1.getPackageName(withDefaults);
    return withDefaults;
}
/**
 * @param {ServerOptions} options used to configure the generated polyserve app
 *     and server.
 * @param {ExpressAppMapper} appMapper optional mapper function which is called
 *     with the generated polyserve app and the options used to generate
 *     it and returns an optional substitution Express app.  This is usually one
 *     that mounts the original app, to add routes or middleware in advance of
 *     polyserve's catch-all routes.
 * @return {Promise} A Promise that completes when the server has started.
 * @deprecated Please use `startServers` instead. This function will be removed
 *     in a future release.
 */
function startServer(options, appMapper) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, _startServer(options, appMapper)];
                case 1: return [2 /*return*/, (_a.sent()).server];
            }
        });
    });
}
exports.startServer = startServer;
function _startServer(options, appMapper) {
    return __awaiter(this, void 0, void 0, function () {
        var app, server, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    options = options || {};
                    assertNodeVersion(options);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 5, , 6]);
                    app = getApp(options);
                    if (!appMapper) return [3 /*break*/, 3];
                    return [4 /*yield*/, appMapper(app, options)];
                case 2:
                    // If the map function doesn't return an app, we should fallback to the
                    // original app, hence the `appMapper(app) || app`.
                    app = (_a.sent()) || app;
                    _a.label = 3;
                case 3: return [4 /*yield*/, startWithApp(options, app)];
                case 4:
                    server = _a.sent();
                    return [2 /*return*/, { app: app, server: server }];
                case 5:
                    e_1 = _a.sent();
                    console.error('ERROR: Server failed to start:', e_1);
                    throw new Error(e_1);
                case 6: return [2 /*return*/];
            }
        });
    });
}
/**
 * Starts one or more web servers, based on the given options and
 * variant bower_components directories that are found in the root dir.
 */
function startServers(options, appMapper) {
    return __awaiter(this, void 0, void 0, function () {
        var variants, serverAndApp;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    options = applyDefaultServerOptions(options);
                    return [4 /*yield*/, findVariants(options)];
                case 1:
                    variants = _a.sent();
                    if (!(variants.length > 0)) return [3 /*break*/, 3];
                    return [4 /*yield*/, startVariants(options, variants, appMapper)];
                case 2: return [2 /*return*/, _a.sent()];
                case 3: return [4 /*yield*/, _startServer(options, appMapper)];
                case 4:
                    serverAndApp = _a.sent();
                    return [2 /*return*/, {
                            options: options,
                            kind: 'mainline',
                            server: serverAndApp.server,
                            app: serverAndApp.app,
                        }];
            }
        });
    });
}
exports.startServers = startServers;
// TODO(usergenic): Variants should support the directory naming convention in
// the .bowerrc instead of hardcoded 'bower_components' form seen here.
function findVariants(options) {
    return __awaiter(this, void 0, void 0, function () {
        var root, filesInRoot, variants;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    root = options.root || process.cwd();
                    return [4 /*yield*/, fs.readdir(root)];
                case 1:
                    filesInRoot = _a.sent();
                    variants = filesInRoot
                        .map(function (f) {
                        var match = f.match("^" + options.componentDir + "-(.*)");
                        return match && { name: match[1], directory: match[0] };
                    })
                        .filter(function (f) { return f != null && f.name !== ''; });
                    return [2 /*return*/, variants];
            }
        });
    });
}
function startVariants(options, variants, appMapper) {
    return __awaiter(this, void 0, void 0, function () {
        var mainlineOptions, mainServer, mainServerInfo, variantServerInfos, _i, variants_1, variant, variantOpts, variantServer, controlServerInfo, servers, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    mainlineOptions = Object.assign({}, options);
                    mainlineOptions.port = 0;
                    return [4 /*yield*/, _startServer(mainlineOptions, appMapper)];
                case 1:
                    mainServer = _a.sent();
                    mainServerInfo = {
                        kind: 'mainline',
                        server: mainServer.server,
                        app: mainServer.app,
                        options: mainlineOptions,
                    };
                    variantServerInfos = [];
                    _i = 0, variants_1 = variants;
                    _a.label = 2;
                case 2:
                    if (!(_i < variants_1.length)) return [3 /*break*/, 5];
                    variant = variants_1[_i];
                    variantOpts = Object.assign({}, options);
                    variantOpts.port = 0;
                    variantOpts.componentDir = variant.directory;
                    return [4 /*yield*/, _startServer(variantOpts, appMapper)];
                case 3:
                    variantServer = _a.sent();
                    variantServerInfos.push({
                        kind: 'variant',
                        variantName: variant.name,
                        dependencyDir: variant.directory,
                        server: variantServer.server,
                        app: variantServer.app,
                        options: variantOpts
                    });
                    _a.label = 4;
                case 4:
                    _i++;
                    return [3 /*break*/, 2];
                case 5: return [4 /*yield*/, startControlServer(options, mainServerInfo, variantServerInfos)];
                case 6:
                    controlServerInfo = _a.sent();
                    servers = [controlServerInfo, mainServerInfo]
                        .concat(variantServerInfos);
                    result = {
                        kind: 'MultipleServers',
                        control: controlServerInfo,
                        mainline: mainServerInfo,
                        variants: variantServerInfos,
                        servers: servers,
                    };
                    return [2 /*return*/, result];
            }
        });
    });
}
function startControlServer(options, mainlineInfo, variantInfos) {
    return __awaiter(this, void 0, void 0, function () {
        var app, indexPath, controlServer, _a;
        var _this = this;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    options = applyDefaultServerOptions(options);
                    app = express();
                    app.get('/api/serverInfo', function (_req, res) {
                        res.contentType('json');
                        res.send(JSON.stringify({
                            packageName: options.packageName,
                            mainlineServer: {
                                port: assertNotString(mainlineInfo.server.address()).port,
                            },
                            variants: variantInfos.map(function (info) {
                                return {
                                    name: info.variantName,
                                    port: assertNotString(info.server.address()).port,
                                };
                            })
                        }));
                        res.end();
                    });
                    indexPath = path.join(__dirname, '..', 'static', 'index.html');
                    app.get('/', function (_req, res) { return __awaiter(_this, void 0, void 0, function () {
                        var indexContents;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    res.contentType('html');
                                    return [4 /*yield*/, fs.readFile(indexPath, 'utf-8')];
                                case 1:
                                    indexContents = _a.sent();
                                    res.send(indexContents);
                                    res.end();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                    _a = {
                        kind: 'control',
                        options: options
                    };
                    return [4 /*yield*/, startWithApp(options, app)];
                case 1:
                    controlServer = (_a.server = _b.sent(),
                        _a.app = app,
                        _a);
                    return [2 /*return*/, controlServer];
            }
        });
    });
}
exports.startControlServer = startControlServer;
function getApp(options) {
    options = applyDefaultServerOptions(options);
    // Preload the h2-push manifest to avoid the cost on first push
    if (options.pushManifestPath) {
        push_1.getPushManifest(options.root, options.pushManifestPath);
    }
    var root = options.root || '.';
    var app = express();
    app.use(compression());
    if (options.additionalRoutes) {
        options.additionalRoutes.forEach(function (handler, route) {
            app.get(route, handler);
        });
    }
    var componentUrl = options.componentUrl;
    var polyserve = make_app_1.makeApp({
        componentDir: options.componentDir,
        packageName: options.packageName,
        root: root,
        headers: options.headers,
    });
    var filePathRegex = /.*\/.+\..{1,}$/;
    if (options.proxy) {
        if (options.proxy.path.startsWith(componentUrl)) {
            console.error("proxy path can not start with " + componentUrl + ".");
            return;
        }
        var escapedPath = options.proxy.path;
        for (var _i = 0, _a = ['*', '?', '+']; _i < _a.length; _i++) {
            var char = _a[_i];
            if (escapedPath.indexOf(char) > -1) {
                console.warn("Proxy path includes character \"" + char + "\"" +
                    "which can cause problems during route matching.");
            }
        }
        if (escapedPath.startsWith('/')) {
            escapedPath = escapedPath.substring(1);
        }
        if (escapedPath.endsWith('/')) {
            escapedPath = escapedPath.slice(0, -1);
        }
        var pathRewrite = {};
        pathRewrite["^/" + escapedPath] = '';
        var apiProxy = httpProxy("/" + escapedPath, {
            target: options.proxy.target,
            changeOrigin: true,
            pathRewrite: pathRewrite,
            logLevel: 'warn',
        });
        app.use("/" + escapedPath + "/", apiProxy);
    }
    app.use('*', custom_elements_es5_adapter_middleware_1.injectCustomElementsEs5Adapter(options.compile));
    app.use('*', compile_middleware_1.babelCompile(options.compile, options.moduleResolution, root, options.packageName, options.componentUrl, options.componentDir));
    if (options.allowOrigin) {
        app.use(cors({ origin: options.allowOrigin }));
    }
    app.use("/" + componentUrl + "/", polyserve);
    // `send` expects files to be specified relative to the given root and as a
    // URL rather than a file system path.
    var entrypoint = options.entrypoint ?
        path_transformers_1.urlFromPath(root, options.entrypoint) :
        'index.html';
    var assetPaths = options.assetPaths || [
        '/*'
    ];
    assetPaths.forEach(function (currentAssetPath) {
        app.get(currentAssetPath, function (req, res) {
            push_1.pushResources(options, req, res);
            var filePath = req.path;
            send(req, filePath, { root: root, index: entrypoint, etag: false, lastModified: false })
                .on('error', function (error) {
                if (error.status === 404 && !filePathRegex.test(filePath)) {
                    // The static file handling middleware failed to find a file on
                    // disk. Serve the entry point HTML file instead of a 404.
                    send(req, entrypoint, { root: root }).pipe(res);
                }
                else {
                    res.status(error.status || 500);
                    res.type('html');
                    res.end(escapeHtml(error.message));
                }
            })
                .pipe(res);
        });
    });
    return app;
}
exports.getApp = getApp;
/**
 * Determines whether a protocol requires HTTPS
 * @param {string} protocol Protocol to evaluate.
 * @returns {boolean}
 */
function isHttps(protocol) {
    return ['https/1.1', 'https', 'h2'].indexOf(protocol) > -1;
}
/**
 * Gets the URLs for the main and component pages
 * @param {ServerOptions} options
 * @returns {{serverUrl: {protocol: string, hostname: string, port: string},
 * componentUrl: url.Url}}
 */
function getServerUrls(options, server) {
    options = applyDefaultServerOptions(options);
    var address = assertNotString(server.address());
    var serverUrl = {
        protocol: isHttps(options.protocol) ? 'https' : 'http',
        hostname: address.address,
        port: String(address.port),
    };
    var componentUrl = Object.assign({}, serverUrl);
    componentUrl.pathname = options.componentUrl + "/" + options.packageName + "/";
    return { serverUrl: serverUrl, componentUrl: componentUrl };
}
exports.getServerUrls = getServerUrls;
/**
 * Asserts that Node version is valid for h2 protocol
 * @param {ServerOptions} options
 */
function assertNodeVersion(options) {
    if (options.protocol === 'h2') {
        var matches = /(\d+)\./.exec(process.version);
        if (matches) {
            var major = Number(matches[1]);
            assert(major >= 5, 'h2 requires ALPN which is only supported in node.js >= 5.0');
        }
    }
}
/**
 * Creates an HTTP(S) server
 * @param app
 * @param {ServerOptions} options
 * @returns {Promise<http.Server>} Promise of server
 */
function createServer(app, options) {
    return __awaiter(this, void 0, void 0, function () {
        var opt, keys;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    opt = { spdy: { protocols: [options.protocol] } };
                    if (!isHttps(options.protocol)) return [3 /*break*/, 2];
                    return [4 /*yield*/, tls_1.getTLSCertificate(options.keyPath, options.certPath)];
                case 1:
                    keys = _a.sent();
                    opt.key = keys.key;
                    opt.cert = keys.cert;
                    return [3 /*break*/, 3];
                case 2:
                    opt.spdy.plain = true;
                    opt.spdy.ssl = false;
                    _a.label = 3;
                case 3: return [2 /*return*/, http.createServer(opt, app)];
            }
        });
    });
}
// Sauce Labs compatible ports taken from
// https://wiki.saucelabs.com/display/DOCS/Sauce+Connect+Proxy+FAQS#SauceConnectProxyFAQS-CanIAccessApplicationsonlocalhost?
// - 80, 443, 888: these ports must have root access
// - 5555, 8080: not forwarded on Android
var SAUCE_PORTS = [
    8081, 8000, 8001, 8003, 8031,
    2000, 2001, 2020, 2109, 2222, 2310, 3000, 3001, 3030, 3210, 3333,
    4000, 4001, 4040, 4321, 4502, 4503, 4567, 5000, 5001, 5050, 5432,
    6000, 6001, 6060, 6666, 6543, 7000, 7070, 7774, 7777, 8765, 8777,
    8888, 9000, 9001, 9080, 9090, 9876, 9877, 9999, 49221, 55001
];
/**
 * Starts an HTTP(S) server serving the given app.
 */
function startWithApp(options, app) {
    return __awaiter(this, void 0, void 0, function () {
        var ports, server, urls;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    options = applyDefaultServerOptions(options);
                    ports = options.port ? [options.port] : SAUCE_PORTS;
                    return [4 /*yield*/, startWithFirstAvailablePort(options, app, ports)];
                case 1:
                    server = _a.sent();
                    urls = getServerUrls(options, server);
                    open_browser_1.openBrowser(options, urls.serverUrl, urls.componentUrl);
                    return [2 /*return*/, server];
            }
        });
    });
}
exports.startWithApp = startWithApp;
function startWithFirstAvailablePort(options, app, ports) {
    return __awaiter(this, void 0, void 0, function () {
        var _i, ports_1, port, server;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    _i = 0, ports_1 = ports;
                    _a.label = 1;
                case 1:
                    if (!(_i < ports_1.length)) return [3 /*break*/, 4];
                    port = ports_1[_i];
                    return [4 /*yield*/, tryStartWithPort(options, app, port)];
                case 2:
                    server = _a.sent();
                    if (server) {
                        return [2 /*return*/, server];
                    }
                    _a.label = 3;
                case 3:
                    _i++;
                    return [3 /*break*/, 1];
                case 4: throw new Error("No available ports. Ports tried: " + JSON.stringify(ports));
            }
        });
    });
}
function tryStartWithPort(options, app, port) {
    return __awaiter(this, void 0, void 0, function () {
        var server;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, createServer(app, options)];
                case 1:
                    server = _a.sent();
                    return [2 /*return*/, new Promise(function (resolve, _reject) {
                            server.listen(port, options.hostname, function () {
                                resolve(server);
                            });
                            server.on('error', function (_err) {
                                resolve(null);
                            });
                        })];
            }
        });
    });
}
// TODO(usergenic): Something changed in the typings of net.Server.address() in
// that it can now return AddressInfo OR string.  I don't know the circumstances
// where the the address() returns a string or how to handle it, so I made this
// assert function when calling on the address to fix compilation errors and
// have a runtime error as soon as the address is fetched.
function assertNotString(value) {
    assert(typeof value !== 'string');
    return value;
}
exports.assertNotString = assertNotString;
