"use strict";
/**
 * @license
 * Copyright (c) 2015 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
exports.__esModule = true;
var url = require("url");
var opn = require("opn");
/**
 * Open the given web page URL. If no browser keyword is provided, `opn` will
 * use the user's default browser.
 */
function openWebPage(url, withBrowser) {
    var openOptions = { app: withBrowser };
    opn(url, openOptions, function (err) {
        if (err) {
            // log error and continue
            console.error("ERROR: Problem launching " +
                ("\"" + (openOptions.app || 'default web browser') + "\"."));
        }
    });
}
/**
 * Opens one or more browsers based on the given options and url params.
 *
 * @param options
 * @param serverUrl
 * @param componentUrl
 */
function openBrowser(options, serverUrl, componentUrl) {
    if (options.open) {
        var openUrl_1;
        if (options.openPath) {
            openUrl_1 = Object.assign({}, serverUrl);
            openUrl_1.pathname = options.openPath;
        }
        else {
            openUrl_1 = Object.assign({}, componentUrl);
        }
        if (!Array.isArray(options.browser)) {
            openWebPage(url.format(openUrl_1));
        }
        else {
            options.browser.forEach(function (browser) {
                openWebPage(url.format(openUrl_1), browser);
            });
        }
    }
}
exports.openBrowser = openBrowser;
