"use strict";
/**
 * @license
 * Copyright (c) 2015 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
exports.__esModule = true;
var fs = require("mz/fs");
var pem = require("pem");
/**
 * Gets the current TLS certificate (from current directory)
 * or generates one if needed
 * @param {string} keyPath path to TLS service key
 * @param {string} certPath path to TLS certificate
 * @returns {Promise<{}>} Promise of {key: string, cert: string}
 */
function getTLSCertificate(keyPath, certPath) {
    return __awaiter(this, void 0, void 0, function () {
        var certObj;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, _readKeyAndCert(keyPath, certPath)];
                case 1:
                    certObj = _a.sent();
                    if (!!certObj) return [3 /*break*/, 4];
                    return [4 /*yield*/, createTLSCertificate()];
                case 2:
                    certObj = _a.sent();
                    if (!(keyPath && certPath)) return [3 /*break*/, 4];
                    return [4 /*yield*/, Promise.all([
                            fs.writeFile(certPath, certObj.cert),
                            fs.writeFile(keyPath, certObj.key)
                        ])];
                case 3:
                    _a.sent();
                    _a.label = 4;
                case 4: return [2 /*return*/, certObj];
            }
        });
    });
}
exports.getTLSCertificate = getTLSCertificate;
function _readKeyAndCert(keyPath, certPath) {
    return __awaiter(this, void 0, void 0, function () {
        var results, cert, key, err_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!keyPath || !certPath) {
                        return [2 /*return*/];
                    }
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, Promise.all([fs.readFile(certPath), fs.readFile(keyPath)])];
                case 2:
                    results = (_a.sent())
                        .map(function (buffer) { return buffer.toString().trim(); });
                    cert = results[0];
                    key = results[1];
                    if (key && cert) {
                        return [2 /*return*/, { cert: cert, key: key }];
                    }
                    return [3 /*break*/, 4];
                case 3:
                    err_1 = _a.sent();
                    // If the cert/key file doesn't exist, generate new TLS certificate
                    if (err_1.code !== 'ENOENT') {
                        throw new Error("cannot read certificate " + err_1);
                    }
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    });
}
/**
 * Generates a TLS certificate for HTTPS
 * @returns {Promise<{}>} Promise of {key: string, cert: string}
 */
function createTLSCertificate() {
    return __awaiter(this, void 0, void 0, function () {
        var keys;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, new Promise(function (resolve, reject) {
                        pem.createCertificate({ days: 365, selfSigned: true }, function (err, keys) {
                            if (err) {
                                reject(err);
                            }
                            else {
                                resolve(keys);
                            }
                        });
                    })];
                case 1:
                    keys = _a.sent();
                    return [2 /*return*/, {
                            cert: keys.certificate,
                            key: keys.serviceKey,
                        }];
            }
        });
    });
}
